package lv.lu.owljgraph;

import lv.lu.owljgraph.edge.OWLDisjointWithEdge;
import lv.lu.owljgraph.edge.OWLDisjointWithView;
import lv.lu.owljgraph.edge.OWLEquivalentClassEdge;
import lv.lu.owljgraph.edge.OWLEquivalentClassView;
import lv.lu.owljgraph.edge.OWLGeneralizationEdge;
import lv.lu.owljgraph.edge.OWLGeneralizationView;
import lv.lu.owljgraph.edge.OWLInverseOfEdge;
import lv.lu.owljgraph.edge.OWLInverseOfView;
import lv.lu.owljgraph.edge.OWLNoteConnectorEdge;
import lv.lu.owljgraph.edge.OWLNoteConnectorView;
import lv.lu.owljgraph.edge.OWLObjectPropertyConnectorEdge;
import lv.lu.owljgraph.edge.OWLObjectPropertyConnectorView;
import lv.lu.owljgraph.edge.OWLSimpleEdge;
import lv.lu.owljgraph.edge.OWLSimpleView;
import lv.lu.owljgraph.vertex.OWLClassCell;
import lv.lu.owljgraph.vertex.OWLClassView;
import lv.lu.owljgraph.vertex.OWLNoteCell;
import lv.lu.owljgraph.vertex.OWLNoteView;
import lv.lu.owljgraph.vertex.OWLObjectPropertyCell;
import lv.lu.owljgraph.vertex.OWLObjectPropertyView;

import org.jgraph.graph.DefaultCellViewFactory;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.VertexView;

public class OWLCellViewFactory extends DefaultCellViewFactory {

    private static final long serialVersionUID = -3094024017072301921L;

    protected EdgeView createEdgeView(Object e) {
        EdgeView edge = null;

        if (e instanceof OWLGeneralizationEdge) {
            edge = new OWLGeneralizationView(e);
        } else if (e instanceof OWLSimpleEdge) {
            edge = new OWLSimpleView(e);
        } else if (e instanceof OWLObjectPropertyConnectorEdge) {
            edge = new OWLObjectPropertyConnectorView(e);
        } else if (e instanceof OWLEquivalentClassEdge) {
            edge = new OWLEquivalentClassView(e);
        } else if (e instanceof OWLDisjointWithEdge) {
            edge = new OWLDisjointWithView(e);
        } else if (e instanceof OWLInverseOfEdge) {
            edge = new OWLInverseOfView(e);
        } else if (e instanceof OWLNoteConnectorEdge) {
            edge = new OWLNoteConnectorView(e);
        } else {
            edge = new EdgeView(e);
        }

        return edge;
    }

    protected VertexView createVertexView(Object v) {
        VertexView vertex = null;

        if (v instanceof OWLObjectPropertyCell) {
            vertex = new OWLObjectPropertyView(v);
        } else if (v instanceof OWLClassCell) {
            vertex = new OWLClassView(v);
        } else if (v instanceof OWLNoteCell) {
            vertex = new OWLNoteView(v);
        } else {
            vertex = new VertexView(v);
        }

        return vertex;
    }

}