package lv.lu.owljgraph.edge;

import java.awt.geom.Point2D;

import lv.lu.owljgraph.OWLGraphConstants;

import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;

public abstract class OWLEdge extends DefaultEdge {

    private static final long serialVersionUID = -9071286918789966108L;
    
    public static final String SIGNATURE = "<<OWLEdge>>"; 
    
    private DefaultGraphCell sourceCell;
    private DefaultGraphCell targetCell;
    
    private String sourceLabel = "";
    private String targetLabel = "";
    private String additionalIdentifier = "";
    
    public OWLEdge() {
        super();
        init();
    }
    
    public OWLEdge(String label) {
        super(label);
        init();
    }
    
    private void init() {
        addPort();
        AttributeMap style = getAttributes();
        OWLGraphConstants.setLabelPosition(style, style.createPoint(new Point2D.Double(OWLGraphConstants.PERMILLE / 2, -20)));
    }
    
    public DefaultGraphCell getSourceCell() {
        return sourceCell;
    }

    public void setSourceCell(DefaultGraphCell sourceCell) {
        this.sourceCell = sourceCell;
        setSource(sourceCell.getChildAt(0));
    }

    public DefaultGraphCell getTargetCell() {
        return targetCell;
    }

    public void setTargetCell(DefaultGraphCell targetCell) {
        this.targetCell = targetCell;
        setTarget(targetCell.getChildAt(0));
    }
    
    public void setSourceLabel(String label) {
        if (label == null) {
            sourceLabel = "";
        } else {
            sourceLabel = label;
        }
        setExtraLabels();
    }
    
    public void setTargetLabel(String label) {
        if (label == null) {
            targetLabel = "";
        } else {
            targetLabel = label;
        }
        setExtraLabels();
    }
    
    protected void setExtraLabels() {
        AttributeMap style = getAttributes();
        Object[] extraLabels = {sourceLabel, targetLabel};
        Point2D[] extraLabelPositions = {
                style.createPoint(new Point2D.Double(OWLGraphConstants.PERMILLE * 0.1, -20)),
                style.createPoint(new Point2D.Double(OWLGraphConstants.PERMILLE * 0.9, -20))
        };
        OWLGraphConstants.setExtraLabels(style, extraLabels);
        OWLGraphConstants.setExtraLabelPositions(style, extraLabelPositions);
    }
    
    public String getAdditionalIdentifier() {
        return additionalIdentifier;
    }

    public void setAdditionalIdentifier(String additionalIdentifier) {
        this.additionalIdentifier = additionalIdentifier;
    }

}