package lv.lu.owljgraph.edge;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.jgraph.graph.EdgeRenderer;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;

public class OWLEdgeRenderer extends EdgeRenderer {

    private static final long serialVersionUID = 6480865305354203137L;
    
    public OWLEdgeRenderer() {
        super();
    }
    
    private boolean isLabelTransformEnabled() {
        return labelTransformEnabled;
    }

    private boolean isLabelTransform(String label) {
        if (!isLabelTransformEnabled()) {
            return false;
        }
        Point2D p = getLabelPosition(view);
        if (p != null && label != null && label.length() > 0) {
            int sw = getLabelWidth(label);
            Point2D p1 = view.getPoint(0);
            Point2D p2 = view.getPoint(view.getPointCount() - 1);
            double length = Math.sqrt((p2.getX() - p1.getX())
                    * (p2.getX() - p1.getX()) + (p2.getY() - p1.getY())
                    * (p2.getY() - p1.getY()));
            if (!(length <= Double.NaN || length < sw)) {
                return true;
            }
        }
        return false;
    }

    private double getLabelAngle(String label) {
        Point2D p = getLabelPosition(view);
        double angle = 0;
        if (p != null && label != null && label.length() > 0) {
            int sw = getLabelWidth(label);
            // Note: For control points you may want to choose other
            // points depending on the segment the label is in.
            Point2D p1 = view.getPoint(0);
            Point2D p2 = view.getPoint(view.getPointCount() - 1);
            // Length of the edge
            double length = Math.sqrt((p2.getX() - p1.getX())
                    * (p2.getX() - p1.getX()) + (p2.getY() - p1.getY())
                    * (p2.getY() - p1.getY()));
            if (!(length <= Double.NaN || length < sw)) { // Label fits into
                // edge's length

                // To calculate projections of edge
                double cos = (p2.getX() - p1.getX()) / length;
                double sin = (p2.getY() - p1.getY()) / length;

                // Determine angle
                angle = Math.acos(cos);
                if (sin < 0) { // Second half
                    angle = 2 * Math.PI - angle;
                }
            }
            if (angle > Math.PI / 2 && angle <= Math.PI * 3 / 2) {
                angle -= Math.PI;
            }
        }
        return angle;
    }

    @Override
    public Dimension getLabelSize(EdgeView view, String label) {
        if (label != null && fontGraphics != null) {
            fontGraphics.setFont(GraphConstants
                    .getFont(view.getAllAttributes()));
            metrics = fontGraphics.getFontMetrics();
            int sw = (int)(getLabelWidth(label) * LABELWIDTHBUFFER);
            int sh = getLabelHeight(label);
            return new Dimension(sw, sh);
        }
        return null;
    }

    @Override
    protected void paintLabel(Graphics g, String label, Point2D p,
            boolean mainLabel) {
        if (labelsEnabled && p != null && label != null && label.length() > 0 && metrics != null) {

            int sw = getLabelWidth(label);
            int sh = getLabelHeight(label);
            
            Graphics2D g2 = (Graphics2D) g;
            boolean applyTransform = isLabelTransform(label);
            double angle = 0;
            int dx = -sw / 2;
            int offset = isMoveBelowZero || applyTransform ? 0 : Math
                    .min(0, (int) (dx + p.getX()));

            g2.translate(p.getX() - offset, p.getY());
            if (applyTransform) {
                angle = getLabelAngle(label);
                g2.rotate(angle);
            }
            if (isOpaque() && mainLabel) {
                g.setColor(getBackground());
                g.fillRect(-sw / 2 - 1, -sh / 2 - 1, sw + 2,
                        sh + 2);
            }
            if (borderColor != null && mainLabel) {
                g.setColor(borderColor);
                g.drawRect(-sw / 2 - 1, -sh / 2 - 1, sw + 2,
                        sh + 2);
            }

            int dy = +metrics.getHeight() / 4;
            g.setColor(fontColor);
            if (applyTransform && borderColor == null && !isOpaque()) {
                // Shift label perpendicularly by the descent so it
                // doesn't cross the line.
                dy = -metrics.getDescent();
            }
            
            String[] labelLines = splitString(label, "\n");
            
            // TODO: remove this dirty hack
            if (labelLines.length > 1) {
                dy -= metrics.getHeight() / 2;
            }
            
            for (String labelLine: labelLines) {
                g.drawString(labelLine, dx, dy);
                dy += metrics.getHeight();
            }
            
            if (applyTransform) {
                // Undo the transform
                g2.rotate(-angle);
            }
            g2.translate(-p.getX() + offset, -p.getY());
        }
    }
    
    protected String[] splitString(String text, String separator) {
        List<String> tokens = new ArrayList<String>();
        StringTokenizer tokenizer = new StringTokenizer(text, separator);
        while (tokenizer.hasMoreTokens()) {
            tokens.add(tokenizer.nextToken());
        }
        String[] result = new String[tokens.size()];
        return tokens.toArray(result);
    }

    protected int getLabelWidth(String label) {
        String[] labelLines = splitString(label, "\n");
        int w = 0;
        for (String labelLine: labelLines) {
            w = Math.max(w, metrics.stringWidth(labelLine));
        }
        return w;
    }

    protected int getLabelHeight(String label) {
        String[] labelLines = splitString(label, "\n");
        int h = 0;
        for (int i = 0; i < labelLines.length; i++) {
            h += metrics.getHeight();
        }
        return h;
    }
    
}