package lv.lu.owljgraph.vertex;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import lv.lu.protege.owl2uml.util.StringUtils;

public class OWLClassCell extends OWLVertexCell {
    
    private static final long serialVersionUID = -8208261461296035240L;
    
    public static final String SIGNATURE = "<<owlClass>>";
    private List<String> restrictions = new ArrayList<String>();
    private Map<String, Set<String>> properties = new HashMap<String, Set<String>>();

    public OWLClassCell(String name) {
        super(name);
    }

    public void addRestriction(String restriction) {
        restrictions.add(restriction);
    }
    
    public void addProperty(String propertyName, String propertyRange) {
        if (properties.get(propertyName) == null) {
            properties.put(propertyName, new HashSet<String>());
        }
        properties.get(propertyName).add(propertyRange);
    }
    
    public void addProperty(String propertyName, Set<String> propertyRanges) {
        properties.put(propertyName, propertyRanges);
    }
    
    public void init() {
        initRestrictions();
        initProperties();
    }
    
    protected void initRestrictions() {
        if (restrictions == null) {
            restrictions = new ArrayList<String>();
        } else {
            restrictions.clear();
        }
    }
    
    protected void initProperties() {
        if (properties == null) {
            properties = new HashMap<String, Set<String>>();
        } else {
            properties.clear();
        }
    }
    
    public List<String> getRestrictions() {
        return restrictions;
    }

    public Map<String, Set<String>> getProperties() {
        return properties;
    }

    public String getPropertiesAreaText() {
        String result = "";
        String r = getRestrictionsString();
        String p = getPropertiesString();
        
        result += r;
        if (r.length() > 0 && p.length() > 0) {
            result += "\n";
        }
        result += p;
        
        return result;
    }

    protected String getRestrictionsString() {
        String result = "";
        for (int i = 0; i < restrictions.size(); i++) {
            if (i > 0) {
                result += "\n";
            }
            result += restrictions.get(i);
        }
        return result;
    }
    
    protected String getPropertiesString() {
        String result = "";
        Boolean isFirstProperty = true;
        for (String propertyName: properties.keySet()) {
            if (!isFirstProperty) {
                result += "\n";
            }
            result += propertyName + ": " + StringUtils.join(properties.get(propertyName).toArray(), ", ");
            isFirstProperty = false;
        }
        return result;
    }
    
}
