package lv.lu.protege.owl2uml;

import java.awt.BorderLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import lv.lu.owljgraph.OWLGraph;
import lv.lu.protege.owl2uml.helper.OWLClassHelper;
import lv.lu.protege.owl2uml.helper.OWLCommonHelper;
import lv.lu.protege.owl2uml.helper.OWLDataPropertyHelper;
import lv.lu.protege.owl2uml.helper.OWLEdgeHelper;
import lv.lu.protege.owl2uml.helper.OWLHelper;
import lv.lu.protege.owl2uml.helper.OWLNoteHelper;
import lv.lu.protege.owl2uml.helper.OWLObjectPropertyHelper;
import lv.lu.protege.owl2uml.helper.OWLRestrictionHelper;
import lv.lu.protege.owl2uml.ui.button.DeserializeButton;
import lv.lu.protege.owl2uml.ui.button.ExportButton;
import lv.lu.protege.owl2uml.ui.button.SerializeButton;
import lv.lu.protege.owl2uml.ui.button.ZoomInButton;
import lv.lu.protege.owl2uml.ui.button.ZoomOutButton;
import lv.lu.protege.owl2uml.ui.checkbox.AssociationsCheckbox;
import lv.lu.protege.owl2uml.ui.checkbox.CommentsCheckbox;
import lv.lu.protege.owl2uml.ui.checkbox.GeneralizationsCheckbox;
import lv.lu.protege.owl2uml.ui.checkbox.OWLCheckBox;
import lv.lu.protege.owl2uml.ui.checkbox.ObjectPropertiesCheckbox;

import org.protege.editor.owl.ui.view.AbstractActiveOntologyViewComponent;
import org.semanticweb.owl.model.OWLOntology;

public class OWL2UML extends AbstractActiveOntologyViewComponent {

    private static final long serialVersionUID = -8336967984998548040L;

    public static final Boolean SHOW_THING_GENERALIZATION = false;
    public static final String THING = "Thing";
    public static final String LITERAL = "Literal";
    
    protected OWLGraph graph;
    protected JScrollPane graphContainer = new JScrollPane();
    protected OWLCheckBox objectPropertiesCheckbox;
    protected OWLCheckBox commentsCheckbox;
    protected OWLCheckBox generalizationsCheckbox;
    protected OWLCheckBox associationsCheckbox;

    @Override
    public void initialiseOntologyView() throws Exception {
        setLayout(new BorderLayout());
        
        graph = new OWLGraph();
        graphContainer.setViewportView(graph);
        add(graphContainer, BorderLayout.CENTER);
        
        objectPropertiesCheckbox = new ObjectPropertiesCheckbox(this);
        commentsCheckbox = new CommentsCheckbox(this);
        generalizationsCheckbox = new GeneralizationsCheckbox(this);
        associationsCheckbox = new AssociationsCheckbox(this);
        
        JPanel topPanel = new JPanel(new BorderLayout());
        add(topPanel, BorderLayout.NORTH);
        
        JPanel leftTopPanel = new JPanel();
        JPanel rightTopPanel = new JPanel();
        topPanel.add(leftTopPanel, BorderLayout.WEST);
        topPanel.add(rightTopPanel, BorderLayout.EAST);

        leftTopPanel.add(new ZoomInButton(this));
        leftTopPanel.add(new ZoomOutButton(this));
        rightTopPanel.add(new JLabel("Show:"));
        rightTopPanel.add(objectPropertiesCheckbox);
        rightTopPanel.add(commentsCheckbox);
        rightTopPanel.add(associationsCheckbox);
        rightTopPanel.add(generalizationsCheckbox);
        
        JPanel bottomPanel = new JPanel(new BorderLayout());
        add(bottomPanel, BorderLayout.SOUTH);
        
        JPanel leftBottomPanel = new JPanel();
        JPanel rightBottomPanel = new JPanel();
        bottomPanel.add(leftBottomPanel, BorderLayout.WEST);
        bottomPanel.add(rightBottomPanel, BorderLayout.EAST);
        
        leftBottomPanel.add(new SerializeButton(this));
        leftBottomPanel.add(new DeserializeButton(this));
        rightBottomPanel.add(new ExportButton(this));

        OWLHelper.init(this);
        updateView();
        graph.layoutGraph();
    }

    public void updateView() {
        updateView(getOWLModelManager().getActiveOntology());
    }

    @Override
    protected void updateView(OWLOntology activeOntology) {
        OWLEdgeHelper.init();
        OWLNoteHelper.init();
        OWLClassHelper.init();
        OWLObjectPropertyHelper.init();
        
        OWLClassHelper.addOWLClasses();
        OWLObjectPropertyHelper.addOWLObjectProperties();
        OWLDataPropertyHelper.addOWLDataProperties();
        OWLRestrictionHelper.addOWLRestrictions();
        
        OWLCommonHelper.updateGraphObjects();
        
        repaint();
    }

    @Override
    public void disposeOntologyView() {
        OWLHelper.dispose();
        graph = null;
    }

    public OWLGraph getGraph() {
        return graph;
    }
    
    public OWLCheckBox getObjectPropertiesCheckbox() {
        return objectPropertiesCheckbox;
    }

    public OWLCheckBox getCommentsCheckbox() {
        return commentsCheckbox;
    }

    public OWLCheckBox getGeneralizationsCheckbox() {
        return generalizationsCheckbox;
    }

    public OWLCheckBox getAssociationsCheckbox() {
        return associationsCheckbox;
    }

}