package lv.lu.protege.owl2uml.helper;

import java.util.HashSet;
import java.util.Set;

import lv.lu.owljgraph.edge.OWLDisjointWithEdge;
import lv.lu.owljgraph.vertex.OWLClassCell;
import lv.lu.owljgraph.vertex.OWLNoteCell;
import lv.lu.protege.owl2uml.OWL2UML;
import lv.lu.protege.owl2uml.util.OWLVertexUtils;

import org.jgraph.graph.DefaultGraphCell;
import org.protege.editor.owl.model.hierarchy.OWLObjectHierarchyProvider;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLObject;

public class OWLClassHelper extends OWLHelper {
    
    private static OWLClass thing;
    private static Set<OWLClass> shownClasses = new HashSet<OWLClass>();
    
    public static void init() {
        shownClasses.clear();
    }
    
    public static void addOWLClasses() {
        init();
        OWLObjectHierarchyProvider<OWLClass> owlClassHierarchyProvider = getOWLClassHierarchyProvider();
        for (OWLClass owlClass : owlClassHierarchyProvider.getRoots()) {
            addOWLClass(null, owlClass);
        }
    }

    private static void addOWLClass(OWLClassCell classParent, OWLClass owlClass) {
        shownClasses.add(owlClass);
        if (OWL2UML.THING.equals(owlClass.toString())) {
            thing = owlClass;
        }
        
        Integer classHash = OWLVertexUtils.getHashCode(owlClass);
        String classId = owlClass.toString();
        OWLClassCell classCell = null;
        
        classCell = getOWLClassCell(classHash, classId);
        classCell.init();
        vertices.updated.add(classCell);
        if (classParent != null && (!OWL2UML.THING.equals(classParent.toString()) || OWL2UML.SHOW_THING_GENERALIZATION)) {
            OWLEdgeHelper.addGeneralizationEdge(classCell, classParent);
        }
        
        processOWLClass(classCell, owlClass);
        
        for (OWLClass child : getOWLClassHierarchyProvider().getChildren(owlClass)) {
            addOWLClass(classCell, child);
        }
    }
    
    private static OWLClassCell getOWLClassCell(Integer classHash, String classId) {
        OWLClassCell classCell = null;
        if (vertices.shown.containsKey(classHash)) {
            classCell = (OWLClassCell) vertices.shown.get(classHash);
        } else {
            classCell = OWLVertexUtils.createOWLClass(classId);
            vertices.shown.put(classHash, classCell);
            vertices.inserted.add(classCell);
        }
        return classCell;
    }
    
    private static void processOWLClass(OWLClassCell classCell, OWLClass owlClass) {
        OWLCommonHelper.processAnnotations(classCell, owlClass);
        processEquivalentClasses(classCell, owlClass);
        processDisjointClasses(classCell, owlClass);
    }
    
    private static void processEquivalentClasses(OWLClassCell classCell, OWLClass owlClass) {
        for (OWLDescription equivalentClass: owlClass.getEquivalentClasses(getActiveOntology())) {
            // Only simple equivalent classes are supported for now
            if (equivalentClass instanceof OWLClass) {
                OWLClassCell equivalentClassCell = getOWLClassCell(
                        OWLVertexUtils.getHashCode((OWLClass) equivalentClass), equivalentClass.toString());
                OWLEdgeHelper.addEquivalentClassEdge(classCell, equivalentClassCell);
            // All unsupported equivalences are renderred as in Protege
            } else {
                addEquivalenceRestriction(classCell, equivalentClass);
            }
        }
    }
    
    private static void processDisjointClasses(OWLClassCell classCell, OWLClass owlClass) {
        Set<OWLDescription> disjointClasses = owlClass.getDisjointClasses(getActiveOntology());
        if (disjointClasses.size() == 1) {
            for (OWLDescription disjointClass: disjointClasses) {
                OWLClassCell DisjointClassCell = getOWLClassCell(
                        OWLVertexUtils.getHashCode((OWLClass) disjointClass), disjointClass.toString());
                OWLEdgeHelper.addDisjointWithEdge(classCell, DisjointClassCell);
            }
        } else if (disjointClasses.size() > 1) {
            // Create note cell
            Set<DefaultGraphCell> connectedClasses = new HashSet<DefaultGraphCell>();
            connectedClasses.add(classCell);
            for (OWLDescription disjointClass: disjointClasses) {
                connectedClasses.add(getOWLClassCell(OWLVertexUtils.getHashCode((OWLClass) disjointClass), disjointClass.toString())); 
            }
            OWLNoteCell noteCell = OWLNoteHelper.addOWLNote(OWLDisjointWithEdge.SIGNATURE, connectedClasses);
            // Add disjoint edges
            OWLEdgeHelper.addNoteConnectorEdge(noteCell, classCell);
            for (OWLDescription disjointClass: disjointClasses) {
                OWLClassCell disjointClassCell = getOWLClassCell(
                        OWLVertexUtils.getHashCode((OWLClass) disjointClass), disjointClass.toString());
                OWLEdgeHelper.addNoteConnectorEdge(noteCell, disjointClassCell);
            }
        }
    }
    
    public static void addEquivalenceRestriction(OWLClassCell owlClassCell, OWLObject restriction) {
        owlClassCell.addRestriction("= {" + parent.getOWLModelManager().getRendering(restriction) + "}");
    }
    
    public static void addSuperRestriction(OWLClassCell owlClassCell, OWLObject restriction) {
        owlClassCell.addRestriction("< {" + parent.getOWLModelManager().getRendering(restriction) + "}");
    }
    
    @SuppressWarnings("deprecation")
    private static OWLObjectHierarchyProvider<OWLClass> getOWLClassHierarchyProvider() {
        return parent.getOWLModelManager().getOWLClassHierarchyProvider();
    }

    public static Set<OWLClass> getShownClasses() {
        return shownClasses;
    }
    
    public static OWLClass getThing() {
        return thing;
    }
    
}