package lv.lu.protege.owl2uml.helper;

import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import lv.lu.owljgraph.edge.OWLEdge;
import lv.lu.owljgraph.vertex.OWLNoteCell;
import lv.lu.owljgraph.vertex.OWLVertexCell;
import lv.lu.protege.owl2uml.util.OWLEdgeUtils;
import lv.lu.protege.owl2uml.util.OWLVertexUtils;

import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphLayoutCache;
import org.semanticweb.owl.model.OWLAnnotation;
import org.semanticweb.owl.model.OWLCommentAnnotation;
import org.semanticweb.owl.model.OWLEntity;

public class OWLCommonHelper extends OWLHelper {

    public static void updateGraphObjects() {
        updateGraphObjects(vertices);
        updateGraphObjects(edges);
        getGraph().repaint();
    }

    public static void updateShownGraphObjects() {
        updateShownGraphObjects(vertices, false, true, false, false);
        updateShownGraphObjects(edges, false, false, false, true);
        parent.updateView();
    }
    
    @SuppressWarnings("unchecked")
    private static void updateGraphObjects(GraphObjects objects) {
        Iterator<Integer> objectIdIterator = objects.shown.keySet().iterator();
        while (objectIdIterator.hasNext()) {
            Integer objectHash = objectIdIterator.next();
            Object object = objects.shown.get(objectHash);
            if (!objects.updated.contains(object)) {
                objects.removed.add(object);
                objectIdIterator.remove();
            }
        }
        GraphLayoutCache layoutCache = getGraph().getGraphLayoutCache();
        layoutCache.insert(objects.inserted.toArray());
        layoutCache.remove(objects.removed.toArray());
        layoutCache.refresh(layoutCache.getMapping(objects.updated.toArray()), false);
        Rectangle2D cellBounds = GraphLayoutCache.getBounds(layoutCache.getMapping(objects.updated.toArray())); 
        getGraph().addOffscreenDirty(cellBounds); 

        objects.inserted.clear();
        objects.removed.clear();
        objects.updated.clear();
    }
    
    @SuppressWarnings("unchecked")
    private static void updateShownGraphObjects(GraphObjects objects, 
            Boolean groups, Boolean vertices, Boolean ports, Boolean edges) {
        Object[] cells = getGraph().getGraphLayoutCache().getCells(groups, vertices, ports, edges);
        objects.shown.clear();
        for (Object cell: cells) {
            Integer cellHash = 0;
            if (cell instanceof OWLNoteCell) {
                cellHash = OWLVertexUtils.getHashCode((OWLNoteCell) cell, getConnectedCells((DefaultGraphCell) cell));
            } else if (cell instanceof OWLVertexCell) {
                cellHash = OWLVertexUtils.getHashCode((OWLVertexCell) cell);
            } else if (cell instanceof OWLEdge) {
                cellHash = OWLEdgeUtils.getHashCode((OWLEdge) cell);
            }
            objects.shown.put(cellHash, cell);
        }
    }
    
    public static Set<DefaultGraphCell> getConnectedCells(DefaultGraphCell cell) {
        Set<DefaultGraphCell> connectedCells = new HashSet<DefaultGraphCell>();
        for (Object edge: DefaultGraphModel.getOutgoingEdges(getGraph().getModel(), cell)) {
            connectedCells.add((DefaultGraphCell) DefaultGraphModel.getTargetVertex(getGraph().getModel(), edge));
        }
        for (Object edge: DefaultGraphModel.getIncomingEdges(getGraph().getModel(), cell)) {
            connectedCells.add((DefaultGraphCell) DefaultGraphModel.getSourceVertex(getGraph().getModel(), edge));
        }
        return connectedCells;
    }
    
    @SuppressWarnings("unchecked")
    public static void processAnnotations(OWLVertexCell cell, OWLEntity entity) {
        for (OWLAnnotation annotation: entity.getAnnotations(getActiveOntology())) {
            // Only comments are supported for now
            if (annotation instanceof OWLCommentAnnotation) {
                String comment = annotation.getAnnotationValueAsConstant().getLiteral();
                OWLNoteCell note = OWLNoteHelper.addOWLNote("<html>" + comment + "</html>", cell);
                OWLEdgeHelper.addNoteConnectorEdge(note, cell);
            }
        }
    }
    
}