package lv.lu.protege.owl2uml.helper;

import java.util.HashSet;
import java.util.Set;

import lv.lu.owljgraph.edge.OWLDisjointWithEdge;
import lv.lu.owljgraph.edge.OWLEdge;
import lv.lu.owljgraph.edge.OWLEquivalentClassEdge;
import lv.lu.owljgraph.edge.OWLGeneralizationEdge;
import lv.lu.owljgraph.edge.OWLInverseOfEdge;
import lv.lu.owljgraph.edge.OWLNoteConnectorEdge;
import lv.lu.owljgraph.edge.OWLObjectPropertyConnectorEdge;
import lv.lu.owljgraph.edge.OWLSimpleEdge;
import lv.lu.protege.owl2uml.util.OWLEdgeUtils;

import org.jgraph.graph.DefaultGraphCell;

public class OWLEdgeHelper extends OWLHelper {

    private static Set<OWLGeneralizationEdge> shownGeneralizations = new HashSet<OWLGeneralizationEdge>();
    private static Set<OWLSimpleEdge> shownSimpleEdges = new HashSet<OWLSimpleEdge>();
    
    public static void init() {
        shownGeneralizations.clear();
        shownSimpleEdges.clear();
    }
    
    public static OWLGeneralizationEdge addGeneralizationEdge(DefaultGraphCell source, DefaultGraphCell target) {
        OWLGeneralizationEdge edge = (OWLGeneralizationEdge) addEdge(OWLGeneralizationEdge.class, source, target);
        shownGeneralizations.add(edge);
        return edge;
    }
    
    public static OWLSimpleEdge addSimpleEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return addSimpleEdge(source, target, "");
    }
    
    public static OWLSimpleEdge addSimpleEdge(DefaultGraphCell source, DefaultGraphCell target, String additionalIdentifier) {
        OWLSimpleEdge edge = (OWLSimpleEdge) addEdge(OWLSimpleEdge.class, source, target, additionalIdentifier);
        shownSimpleEdges.add(edge);
        return edge;
    }
    
    public static OWLObjectPropertyConnectorEdge addObjectPropertyConnectorEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLObjectPropertyConnectorEdge) addEdge(OWLObjectPropertyConnectorEdge.class, source, target);
    }
    
    public static OWLEquivalentClassEdge addEquivalentClassEdge(DefaultGraphCell source, DefaultGraphCell target) {
        if (edges.shown.containsKey(OWLEdgeUtils.getHashCode(OWLEquivalentClassEdge.class, target, source))) {
            return null;
        }
        return (OWLEquivalentClassEdge) addEdge(OWLEquivalentClassEdge.class, source, target);
    }
    
    public static OWLDisjointWithEdge addDisjointWithEdge(DefaultGraphCell source, DefaultGraphCell target) {
        if (edges.shown.containsKey(OWLEdgeUtils.getHashCode(OWLDisjointWithEdge.class, target, source))) {
            return null;
        }
        return (OWLDisjointWithEdge) addEdge(OWLDisjointWithEdge.class, source, target);
    }
    
    public static OWLInverseOfEdge addInverseOfEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLInverseOfEdge) addEdge(OWLInverseOfEdge.class, source, target);
    }
    
    public static OWLNoteConnectorEdge addNoteConnectorEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLNoteConnectorEdge) addEdge(OWLNoteConnectorEdge.class, source, target);
    }
    
    @SuppressWarnings("unchecked")
    private static OWLEdge addEdge(Class edgeClass, DefaultGraphCell source, DefaultGraphCell target) {
        return addEdge(edgeClass, source, target, "");
    }
    
    @SuppressWarnings("unchecked")
    private static OWLEdge addEdge(Class edgeClass, DefaultGraphCell source, DefaultGraphCell target, String additionalIdentifier) {
        if (source == null || target == null) {
            return null;
        }
        Integer edgeHash = OWLEdgeUtils.getHashCode(edgeClass, source, target, additionalIdentifier);
        OWLEdge edge = null;
        if (edges.shown.containsKey(edgeHash)) {
            edge = edges.shown.get(edgeHash);
        } else {
            edge = OWLEdgeUtils.createEdge(edgeClass, source, target, additionalIdentifier);
            edges.shown.put(edgeHash, edge);
            edges.inserted.add(edge);
        }
        edges.updated.add(edge);
        return edge;
    }
    
    public static Set<OWLGeneralizationEdge> getShownGeneralizations() {
        return shownGeneralizations;
    }

    public static Set<OWLSimpleEdge> getShownSimpleEdges() {
        return shownSimpleEdges;
    }

}
