package lv.lu.protege.owl2uml.helper;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import lv.lu.owljgraph.edge.OWLDisjointWithEdge;
import lv.lu.owljgraph.edge.OWLEdge;
import lv.lu.owljgraph.vertex.OWLNoteCell;
import lv.lu.owljgraph.vertex.OWLObjectPropertyCell;
import lv.lu.owljgraph.vertex.OWLVertexCell;
import lv.lu.protege.owl2uml.util.OWLVertexUtils;

import org.jgraph.graph.DefaultGraphCell;
import org.protege.editor.owl.model.hierarchy.OWLObjectHierarchyProvider;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLObjectPropertyExpression;

public class OWLObjectPropertyHelper extends OWLHelper {
    
    private static Set<OWLObjectPropertyCell> shownObjectProperties = new HashSet<OWLObjectPropertyCell>();
    
    public static void init() {
        shownObjectProperties.clear();
    }
    
    public static void addOWLObjectProperties() {
        init();
        OWLObjectHierarchyProvider<OWLObjectProperty> hierarchyProvider = getOWLObjectPropertyHierarchyProvider();
        for (OWLObjectProperty objectProperty: hierarchyProvider.getRoots()) {
            addOWLObjectProperty(null, objectProperty);
        }
    }
    
    private static void addOWLObjectProperty(OWLObjectPropertyCell propertyParent, OWLObjectProperty objectProperty) {
        Integer objectPropertyHash = OWLVertexUtils.getHashCode(objectProperty);
        String objectPropertyId = objectProperty.toString();
        
        OWLObjectPropertyCell objectPropertyCell = getObjectPropertyCell(objectPropertyHash, objectPropertyId);
        shownObjectProperties.add(objectPropertyCell);
        vertices.updated.add(objectPropertyCell);
        OWLEdgeHelper.addGeneralizationEdge(objectPropertyCell, propertyParent);
        
        processOWLObjectProperty(objectPropertyCell, objectProperty);
        
        OWLObjectHierarchyProvider<OWLObjectProperty> hierarchyProvider = getOWLObjectPropertyHierarchyProvider();
        for (OWLObjectProperty child : hierarchyProvider.getChildren(objectProperty)) {
            addOWLObjectProperty(objectPropertyCell, child);
        }
    }

    private static OWLObjectPropertyCell getObjectPropertyCell(Integer propertyHash, String propertyId) {
        OWLObjectPropertyCell propertyCell = null;
        if (vertices.shown.containsKey(propertyHash)) {
            propertyCell = (OWLObjectPropertyCell) vertices.shown.get(propertyHash);
        } else {
            propertyCell = OWLVertexUtils.createOWLObjectProperty(propertyId);
            vertices.shown.put(propertyHash, propertyCell);
            vertices.inserted.add(propertyCell);
        }
        return propertyCell;
    }
    
    private static void processOWLObjectProperty(OWLObjectPropertyCell objectPropertyCell, OWLObjectProperty objectProperty) {
        OWLCommonHelper.processAnnotations(objectPropertyCell, objectProperty);
        processDomainsAndRanges(objectPropertyCell, objectProperty);
        processEquivalentProperties(objectPropertyCell, objectProperty);
        processDisjointProperties(objectPropertyCell, objectProperty);
        processInverseProperties(objectPropertyCell, objectProperty);
    }
    
    private static void processDomainsAndRanges(OWLObjectPropertyCell objectPropertyCell, OWLObjectProperty objectProperty) {
        List <OWLEdge> domainRangeEdgeList = new ArrayList<OWLEdge>();
        
        Set<OWLDescription> domains = objectProperty.getDomains(getActiveOntology());
        Set<OWLDescription> ranges = objectProperty.getRanges(getActiveOntology());
        
        // If there are no domains or ranges, then assuming Thing
        if ((domains.isEmpty() && !ranges.isEmpty()) || (!domains.isEmpty() && ranges.isEmpty())) {
            if (domains.isEmpty()) {
                domains.add(OWLClassHelper.getThing());
            }
            if (ranges.isEmpty()) {
                ranges.add(OWLClassHelper.getThing());
            }
        }
        
        for (OWLDescription domain: domains) {
            if (domain instanceof OWLClass) {
                for (OWLDescription range: ranges) {
                    if (range instanceof OWLClass) {
                        OWLVertexCell domainCell = vertices.shown.get(OWLVertexUtils.getHashCode((OWLClass) domain));
                        OWLVertexCell rangeCell = vertices.shown.get(OWLVertexUtils.getHashCode((OWLClass) range));
                        
                        OWLEdge domainRangeEdge = OWLEdgeHelper.addSimpleEdge(domainCell, rangeCell, objectProperty.toString());
                        domainRangeEdge.setUserObject("+" + objectProperty.toString());
                        OWLEdgeHelper.addObjectPropertyConnectorEdge(domainRangeEdge, objectPropertyCell);
                        
                        domainRangeEdgeList.add(domainRangeEdge);
                    }
                }
            }
        }
        // Cardinality
        String sourceLabel = null;
        String targetLabel = null;
        if (objectProperty.isInverseFunctional(getActiveOntology())) {
            sourceLabel = "1";
        }
        if (objectProperty.isFunctional(getActiveOntology())) {
            targetLabel = "1";
        }
        for (OWLEdge domainRangeEdge: domainRangeEdgeList) {
            domainRangeEdge.setSourceLabel(sourceLabel);
            domainRangeEdge.setTargetLabel(targetLabel);
        }
    }
    
    private static void processEquivalentProperties(OWLObjectPropertyCell objectPropertyCell, OWLObjectProperty objectProperty) {
        // Not supported for now
        // Rendering as in Protege
        for (OWLObjectPropertyExpression equivalentProperty: objectProperty.getEquivalentProperties(getActiveOntology())) {
            OWLClassHelper.addEquivalenceRestriction(objectPropertyCell, equivalentProperty);
        }
    }
    
    private static void processDisjointProperties(OWLObjectPropertyCell objectPropertyCell, OWLObjectProperty objectProperty) {
        Set<OWLObjectPropertyExpression> disjointProperties = objectProperty.getDisjointProperties(getActiveOntology());
        if (disjointProperties.size() == 1) {
            for (OWLObjectPropertyExpression disjointProperty: disjointProperties) {
                OWLObjectPropertyCell disjointObjectPropertyCell = getObjectPropertyCell(
                        OWLVertexUtils.getHashCode((OWLObjectProperty) disjointProperty), disjointProperty.toString());
                OWLEdgeHelper.addDisjointWithEdge(objectPropertyCell, disjointObjectPropertyCell);
            }
        } else if (disjointProperties.size() > 1) {
            // Create note cell
            Set<DefaultGraphCell> connectedClasses = new HashSet<DefaultGraphCell>();
            connectedClasses.add(objectPropertyCell);
            for (OWLObjectPropertyExpression disjointProperty: disjointProperties) {
                connectedClasses.add(getObjectPropertyCell(
                        OWLVertexUtils.getHashCode((OWLObjectProperty) disjointProperty), disjointProperty.toString())); 
            }
            OWLNoteCell noteCell = OWLNoteHelper.addOWLNote(OWLDisjointWithEdge.SIGNATURE, connectedClasses);
            // Add disjoint edges
            OWLEdgeHelper.addNoteConnectorEdge(noteCell, objectPropertyCell);
            for (OWLObjectPropertyExpression disjointProperty: disjointProperties) {
                OWLObjectPropertyCell disjointObjectPropertyCell = getObjectPropertyCell(
                        OWLVertexUtils.getHashCode((OWLObjectProperty) disjointProperty), disjointProperty.toString());
                OWLEdgeHelper.addNoteConnectorEdge(noteCell, disjointObjectPropertyCell);
            }
        }
    }
    
    private static void processInverseProperties(OWLObjectPropertyCell objectPropertyCell, OWLObjectProperty objectProperty) {
        for (OWLObjectPropertyExpression inverseProperty: objectProperty.getInverses(getActiveOntology())) {
            OWLObjectPropertyCell inverseCell = getObjectPropertyCell(
                    OWLVertexUtils.getHashCode((OWLObjectProperty) inverseProperty), inverseProperty.toString());
            OWLEdgeHelper.addInverseOfEdge(objectPropertyCell, inverseCell);
        }
    }

    @SuppressWarnings("deprecation")
    private static OWLObjectHierarchyProvider<OWLObjectProperty> getOWLObjectPropertyHierarchyProvider() {
        return parent.getOWLModelManager().getOWLObjectPropertyHierarchyProvider();
    }

    public static Set<OWLObjectPropertyCell> getShownObjectProperties() {
        return shownObjectProperties;
    }
    
}