package lv.lu.protege.owl2uml.helper;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import lv.lu.owljgraph.edge.OWLEdge;
import lv.lu.owljgraph.edge.OWLSimpleEdge;
import lv.lu.owljgraph.vertex.OWLClassCell;
import lv.lu.owljgraph.vertex.OWLVertexCell;
import lv.lu.protege.owl2uml.OWL2UML;
import lv.lu.protege.owl2uml.util.OWLEdgeUtils;
import lv.lu.protege.owl2uml.util.OWLVertexUtils;

import org.semanticweb.owl.model.OWLAnonymousDescription;
import org.semanticweb.owl.model.OWLCardinalityRestriction;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLDataCardinalityRestriction;
import org.semanticweb.owl.model.OWLDataExactCardinalityRestriction;
import org.semanticweb.owl.model.OWLDataMaxCardinalityRestriction;
import org.semanticweb.owl.model.OWLDataMinCardinalityRestriction;
import org.semanticweb.owl.model.OWLDataProperty;
import org.semanticweb.owl.model.OWLDataRange;
import org.semanticweb.owl.model.OWLDescription;
import org.semanticweb.owl.model.OWLObjectAllRestriction;
import org.semanticweb.owl.model.OWLObjectCardinalityRestriction;
import org.semanticweb.owl.model.OWLObjectExactCardinalityRestriction;
import org.semanticweb.owl.model.OWLObjectMaxCardinalityRestriction;
import org.semanticweb.owl.model.OWLObjectMinCardinalityRestriction;
import org.semanticweb.owl.model.OWLObjectProperty;
import org.semanticweb.owl.model.OWLPropertyRange;
import org.semanticweb.owl.model.OWLQuantifiedRestriction;

public class OWLRestrictionHelper extends OWLHelper {
    
    public static void addOWLRestrictions() {
        for (OWLClass owlClass: OWLClassHelper.getShownClasses()) {
            processOWLClassRestrictions(owlClass);
        }
    }
    
    @SuppressWarnings("unchecked")
    private static void processOWLClassRestrictions(OWLClass owlClass) {
        
        OWLClassCell owlClassCell = (OWLClassCell) vertices.shown.get(OWLVertexUtils.getHashCode(owlClass));
        Set<OWLDescription> restrictions = owlClass.getSuperClasses(getActiveOntology());
        
        Map<OWLObjectProperty, Set<OWLQuantifiedRestriction>> objectPropertyCardinalities 
                = new HashMap<OWLObjectProperty, Set<OWLQuantifiedRestriction>>();
        Map<OWLDataProperty, Set<OWLQuantifiedRestriction>> dataPropertyCardinalities 
                = new HashMap<OWLDataProperty, Set<OWLQuantifiedRestriction>>();
        
        for (OWLDescription restriction: restrictions) {
            if (restriction instanceof OWLAnonymousDescription) {
                
                // Supported restrictions
                if (restriction instanceof OWLQuantifiedRestriction) {
                    
                    OWLQuantifiedRestriction r = (OWLQuantifiedRestriction) restriction;
                    Boolean isOWLClassFiller = (r.getFiller() instanceof OWLClass);
                    
                    // Object property allValuesFrom simple restrictions
                    if (restriction instanceof OWLObjectAllRestriction && isOWLClassFiller) {
                        
                        OWLObjectProperty objectProperty = (OWLObjectProperty) r.getProperty();
                        processObjectAllRestriction(owlClass, objectProperty, (OWLObjectAllRestriction) r);
                        
                    // Object property cardinality restrictions
                    } else if (restriction instanceof OWLObjectCardinalityRestriction && isOWLClassFiller) {
                        
                        // Grouping object property cardinality restrictions by object property
                        OWLObjectProperty objectProperty = (OWLObjectProperty) r.getProperty();
                        if (!objectPropertyCardinalities.keySet().contains(objectProperty)) {
                            objectPropertyCardinalities.put(objectProperty, new HashSet<OWLQuantifiedRestriction>());
                        }
                        objectPropertyCardinalities.get(objectProperty).add(r);
                        
                    // Data property cardinality restrictions
                    } else if (restriction instanceof OWLDataCardinalityRestriction) {
                        
                        // Grouping data property cardinality restrictions by object property
                        OWLDataProperty dataProperty = (OWLDataProperty) r.getProperty();
                        if (!dataPropertyCardinalities.keySet().contains(dataProperty)) {
                            dataPropertyCardinalities.put(dataProperty, new HashSet<OWLQuantifiedRestriction>());
                        }
                        dataPropertyCardinalities.get(dataProperty).add(r);
                        
                    // All others unsupported restrictions
                    } else {
                        
                        OWLClassHelper.addSuperRestriction(owlClassCell, restriction);
                        
                    }
                    
                // Unsupported restrictions
                } else {

                    OWLClassHelper.addSuperRestriction(owlClassCell, restriction);

                }
                
            } 
        }
        
        // Processing object property cardinality restrictions
        for (OWLObjectProperty objectProperty: objectPropertyCardinalities.keySet()) {
            Map<OWLPropertyRange, Set<OWLQuantifiedRestriction>> groupedByFiller 
                    = groupRestrictionsByFiller(objectPropertyCardinalities.get(objectProperty));
            for (OWLPropertyRange filler: groupedByFiller.keySet()) {
                processObjectCardinalityRestrictions(owlClass, objectProperty, (OWLClass) filler, groupedByFiller.get(filler));
            }
        }
        
        // Processing data property cardinality restrictions
        for (OWLDataProperty dataProperty: dataPropertyCardinalities.keySet()) {
            Map<OWLPropertyRange, Set<OWLQuantifiedRestriction>> groupedByFiller 
                    = groupRestrictionsByFiller(dataPropertyCardinalities.get(dataProperty));
            for (OWLPropertyRange filler: groupedByFiller.keySet()) {
                processDataCardinalityRestrictions(owlClass, dataProperty, (OWLDataRange) filler, groupedByFiller.get(filler));
            }
        }
        
    }
    
    private static void processObjectAllRestriction(OWLClass owlClass, 
            OWLObjectProperty objectProperty, OWLObjectAllRestriction restriction) {
        OWLClass restrictionFiller = (OWLClass) restriction.getFiller();
        addDomainRangeEdge(owlClass, restrictionFiller, objectProperty);
    }
    
    @SuppressWarnings("unchecked")
    private static void processObjectCardinalityRestrictions(OWLClass owlClass, 
            OWLObjectProperty objectProperty, OWLClass restrictionFiller, Set restrictions) {
        String cardinalityString = getCardinalityString(restrictions);
        if (restrictionFiller.toString().equals(OWL2UML.THING)) {
            for (OWLDescription range: objectProperty.getRanges(getActiveOntology())) {
                if (range instanceof OWLClass) {
                    OWLEdge domainRangeEdge = addDomainRangeEdge(owlClass, (OWLClass) range, objectProperty);
                    domainRangeEdge.setTargetLabel(cardinalityString);
                }
            }
        } else {
            OWLEdge domainRangeEdge = addDomainRangeEdge(owlClass, restrictionFiller, objectProperty);
            domainRangeEdge.setTargetLabel(cardinalityString);
        }
    }
    
    @SuppressWarnings("unchecked")
    private static void processDataCardinalityRestrictions(OWLClass owlClass, 
            OWLDataProperty dataProperty, OWLDataRange restrictionFiller, Set restrictions) {
        OWLClassCell owlClassCell = (OWLClassCell) vertices.shown.get(OWLVertexUtils.getHashCode(owlClass));
        String restrictionRange = parent.getOWLModelManager().getRendering(restrictionFiller);
        String cardinalityString = " [" + getCardinalityString(restrictions) + "]";

        // TODO Refactor
        if (restrictionRange.equals(OWL2UML.LITERAL)) {
            Set<String> newRanges = new HashSet<String>();
            for (OWLDataRange range: dataProperty.getRanges(getActiveOntology())) {
                String rangeString = parent.getOWLModelManager().getRendering(range);
                newRanges.add(rangeString + cardinalityString);
            }
            owlClassCell.addProperty(dataProperty.toString(), newRanges);
        } else {
            owlClassCell.addProperty(dataProperty.toString(), restrictionRange);
            owlClassCell.getProperties().get(dataProperty.toString()).remove(restrictionRange);
            owlClassCell.getProperties().get(dataProperty.toString()).add(restrictionRange + cardinalityString);
        }
        
    }
    
    @SuppressWarnings("unchecked")
    private static Map<OWLPropertyRange, Set<OWLQuantifiedRestriction>> groupRestrictionsByFiller(Set<OWLQuantifiedRestriction> restrictions) {
        
        Map<OWLPropertyRange, Set<OWLQuantifiedRestriction>> groupedRestrictions 
                = new HashMap<OWLPropertyRange, Set<OWLQuantifiedRestriction>>();
        
        for (OWLQuantifiedRestriction r: restrictions) {
            if (!groupedRestrictions.keySet().contains(r.getFiller())) {
                groupedRestrictions.put(r.getFiller(), new HashSet<OWLQuantifiedRestriction>());
            }
            groupedRestrictions.get(r.getFiller()).add(r);
        }
        
        return groupedRestrictions;
    }
    
    @SuppressWarnings("unchecked")
    private static String getCardinalityString(Set restrictions) {
        String exactCardinality = null;
        String minCardinality = "";
        String maxCardinality = "";
        
        for (Object o: restrictions) {
            OWLCardinalityRestriction r = (OWLCardinalityRestriction) o;
            if (r instanceof OWLObjectExactCardinalityRestriction
                    || r instanceof OWLDataExactCardinalityRestriction) {
                exactCardinality = String.valueOf(r.getCardinality());
            } else if (r instanceof OWLObjectMinCardinalityRestriction
                    || r instanceof OWLDataMinCardinalityRestriction) {
                minCardinality = String.valueOf(r.getCardinality());
            } else if (r instanceof OWLObjectMaxCardinalityRestriction
                    || r instanceof OWLDataMaxCardinalityRestriction) {
                maxCardinality = String.valueOf(r.getCardinality());
            } 
        }
        
        String cardinalityString = "";
        if (exactCardinality != null) {
            cardinalityString = exactCardinality;
        } else {
            cardinalityString = minCardinality + ".." + maxCardinality;
        }
        
        return cardinalityString;
    }
    
    private static OWLEdge addDomainRangeEdge(OWLClass domain, OWLClass range, OWLObjectProperty objectProperty) {
        OWLVertexCell domainCell = vertices.shown.get(OWLVertexUtils.getHashCode(domain));
        OWLVertexCell rangeCell = vertices.shown.get(OWLVertexUtils.getHashCode(range));
        OWLVertexCell objectPropertyCell = vertices.shown.get(OWLVertexUtils.getHashCode(objectProperty));
        
        String edgeLabel = "+" + objectProperty.toString();
        // If there is no such edge yet => {redefines}
        Integer edgeHash = OWLEdgeUtils.getHashCode(OWLSimpleEdge.class, domainCell, rangeCell, objectProperty.toString());
        if (!edges.shown.containsKey(edgeHash) || !edges.updated.contains(edges.shown.get(edgeHash))) {
            edgeLabel += "\n{redefines " + objectProperty.toString() +"}";
        }
        
        OWLEdge domainRangeEdge = OWLEdgeHelper.addSimpleEdge(domainCell, rangeCell, objectProperty.toString());
        domainRangeEdge.setUserObject(edgeLabel);
        
        OWLEdgeHelper.addObjectPropertyConnectorEdge(domainRangeEdge, objectPropertyCell);
        
        return domainRangeEdge;
    }
    
}