package lv.lu.protege.owl2uml.ui.button;

import java.awt.event.ActionEvent;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import javax.swing.JFileChooser;

import lv.lu.owljgraph.OWLCellViewFactory;
import lv.lu.owljgraph.OWLGraph;
import lv.lu.protege.owl2uml.OWL2UML;
import lv.lu.protege.owl2uml.bean.GraphLayout;

import org.jgraph.graph.GraphLayoutCache;

public class SerializeButton extends OWLButton {

    private static final long serialVersionUID = -3179121248160881697L;

    public SerializeButton(OWL2UML parent) {
        super("Save layout", "serialize", parent);
    }

    public void actionPerformed(ActionEvent event) {
        if ("serialize".equals(event.getActionCommand())) {
            File out = getSerializeFile();
            if (out != null) {
                saveCheckboxStates();
                selectAllCheckboxes();
                
                OWLGraph graph = parent.getGraph();
                
                GraphLayoutCache layoutCache = graph.getGraphLayoutCache();
                GraphLayoutCache layoutCacheTmp = new GraphLayoutCache(graph.getModel(), new OWLCellViewFactory(), true);
                graph.setGraphLayoutCache(layoutCacheTmp);
                
                graph.clearSelection();

                GraphLayout graphLayout = createGraphLayout(layoutCache);
                
                try {
                    ObjectOutputStream outStream = new ObjectOutputStream(
                            new BufferedOutputStream(new FileOutputStream(out)));
                    outStream.writeObject(graphLayout);
                    outStream.flush();
                    outStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    graph.setGraphLayoutCache(layoutCache);
                    graph.getGraphLayoutCache().reload();
                    graph.repaint();
                }
                
                restoreCheckboxStates();
            }
        }
    }

    public File getSerializeFile() {
        final JFileChooser fc = new JFileChooser();
        if (fc.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFile();
        }
        return null;
    }
    
    protected GraphLayout createGraphLayout(GraphLayoutCache layoutCache) {
        GraphLayout layout = new GraphLayout();
        layout.setLayoutCache(layoutCache);
        layout.setIsObjectPropertiesCheckboxSelected(isObjectPropertiesCheckboxSelected);
        layout.setIsCommentsCheckboxSelected(isCommentsCheckboxSelected);
        layout.setIsGeneralizationsCheckboxSelected(isGeneralizationsCheckboxSelected);
        layout.setIsAssociationsCheckboxSelected(isAssociationsCheckboxSelected);
        return layout;
    }

}