package lv.lu.protege.owl2uml.util;

import java.lang.reflect.Constructor;

import lv.lu.owljgraph.edge.OWLDisjointWithEdge;
import lv.lu.owljgraph.edge.OWLEdge;
import lv.lu.owljgraph.edge.OWLEquivalentClassEdge;
import lv.lu.owljgraph.edge.OWLGeneralizationEdge;
import lv.lu.owljgraph.edge.OWLInverseOfEdge;
import lv.lu.owljgraph.edge.OWLNoteConnectorEdge;
import lv.lu.owljgraph.edge.OWLObjectPropertyConnectorEdge;
import lv.lu.owljgraph.edge.OWLSimpleEdge;

import org.jgraph.graph.DefaultGraphCell;

public class OWLEdgeUtils {

    public static OWLGeneralizationEdge createGeneralizationEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLGeneralizationEdge) createEdge(OWLGeneralizationEdge.class, source, target);
    }
    
    public static OWLSimpleEdge createSimpleEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLSimpleEdge) createEdge(OWLSimpleEdge.class, source, target);
    }
    
    public static OWLObjectPropertyConnectorEdge createObjectPropertyConnectorEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLObjectPropertyConnectorEdge) createEdge(OWLObjectPropertyConnectorEdge.class, source, target);
    }
    
    public static OWLEquivalentClassEdge createEquivalentClassEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLEquivalentClassEdge) createEdge(OWLEquivalentClassEdge.class, source, target);
    }
    
    public static OWLDisjointWithEdge createDisjointWithEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLDisjointWithEdge) createEdge(OWLDisjointWithEdge.class, source, target);
    }
    
    public static OWLInverseOfEdge createInverseOfEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLInverseOfEdge) createEdge(OWLInverseOfEdge.class, source, target);
    }
    
    public static OWLNoteConnectorEdge createNoteConnectorEdge(DefaultGraphCell source, DefaultGraphCell target) {
        return (OWLNoteConnectorEdge) createEdge(OWLNoteConnectorEdge.class, source, target);
    }
    
    @SuppressWarnings("unchecked")
    public static OWLEdge createEdge(Class edgeClass, DefaultGraphCell source, DefaultGraphCell target) {
        return createEdge(edgeClass, source, target, "");
    }
    
    @SuppressWarnings("unchecked")
    public static OWLEdge createEdge(Class edgeClass, DefaultGraphCell source, DefaultGraphCell target, String additionalIdentifier) {
        OWLEdge edge = null;
        try {
            Constructor edgeConstructor = edgeClass.getConstructor(new Class[] {});
            edge = (OWLEdge) edgeConstructor.newInstance(new Object[] {});
            edge.setSourceCell(source);
            edge.setTargetCell(target);
            edge.setAdditionalIdentifier(additionalIdentifier);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return edge;
    }
    
    @SuppressWarnings("unchecked")
    public static Integer getHashCode(Class edgeClass, DefaultGraphCell source, DefaultGraphCell target) {
        return getHashCode(edgeClass, source, target, "");
    }
    
    @SuppressWarnings("unchecked")
    public static Integer getHashCode(Class edgeClass, DefaultGraphCell source, DefaultGraphCell target, String additionalIdentifier) {
        return StringUtils.getHashCode(getSignature(edgeClass) + OWLGraphUtils.getId(source) + OWLGraphUtils.getId(target) + additionalIdentifier);
    }
    
    public static Integer getHashCode(OWLEdge edge) {
        return StringUtils.getHashCode(getId(edge));
    }
    
    public static String getId(OWLEdge edge) {
        return getSignature(edge) + OWLGraphUtils.getId(edge.getSourceCell()) + OWLGraphUtils.getId(edge.getTargetCell()) + edge.getAdditionalIdentifier();
    }
    
    protected static String getSignature(OWLEdge edge) {
        return getSignature(edge.getClass());
    }
    
    @SuppressWarnings("unchecked")
    protected static String getSignature(Class edgeClass) {
        if (edgeClass == OWLGeneralizationEdge.class) {
            return OWLGeneralizationEdge.SIGNATURE;
        } else if (edgeClass == OWLSimpleEdge.class) {
            return OWLSimpleEdge.SIGNATURE;
        } else if (edgeClass == OWLObjectPropertyConnectorEdge.class) {
            return OWLObjectPropertyConnectorEdge.SIGNATURE;
        } else if (edgeClass == OWLEquivalentClassEdge.class) {
            return OWLEquivalentClassEdge.SIGNATURE;
        } else if (edgeClass == OWLDisjointWithEdge.class) {
            return OWLDisjointWithEdge.SIGNATURE;
        } else if (edgeClass == OWLInverseOfEdge.class) {
            return OWLInverseOfEdge.SIGNATURE;
        } else if (edgeClass == OWLNoteConnectorEdge.class) {
            return OWLNoteConnectorEdge.SIGNATURE;
        } else {
            return OWLEdge.SIGNATURE;
        }
    }
    
}
