package lv.lu.protege.owl2uml.util;

import java.awt.geom.Rectangle2D;
import java.util.Set;

import lv.lu.owljgraph.OWLGraphConstants;
import lv.lu.owljgraph.vertex.OWLClassCell;
import lv.lu.owljgraph.vertex.OWLNoteCell;
import lv.lu.owljgraph.vertex.OWLObjectPropertyCell;
import lv.lu.owljgraph.vertex.OWLVertexCell;

import org.jgraph.graph.DefaultGraphCell;
import org.semanticweb.owl.model.OWLClass;
import org.semanticweb.owl.model.OWLObjectProperty;

public class OWLVertexUtils {

    public static OWLClassCell createOWLClass(String name) {
        OWLClassCell classCell = new OWLClassCell(name);
        setProperties(classCell);
        return classCell;
    }
    
    public static OWLObjectPropertyCell createOWLObjectProperty(String name) {
        OWLObjectPropertyCell objectPropertyCell = new OWLObjectPropertyCell(name);
        setProperties(objectPropertyCell);
        return objectPropertyCell;
    }
    
    public static OWLNoteCell createOWLNote(String name) {
        OWLNoteCell noteCell = new OWLNoteCell(name);
        setProperties(noteCell);
        return noteCell;
    }
    
    protected static void setProperties(OWLVertexCell vertexCell) {
        OWLGraphConstants.setResize(vertexCell.getAttributes(), true);
        OWLGraphConstants.setBounds(vertexCell.getAttributes(), new Rectangle2D.Double(0, 0, 0, 0));
    }
    
    public static Integer getHashCode(OWLClass owlClass) {
        return StringUtils.getHashCode(getId(owlClass));
    }
    
    public static Integer getHashCode(OWLObjectProperty objectProperty) {
        return StringUtils.getHashCode(getId(objectProperty));
    }
    
    public static Integer getHashCode(OWLVertexCell vertex) {
        return StringUtils.getHashCode(getId(vertex));
    }
    
    public static Integer getHashCode(OWLNoteCell noteCell, Set<DefaultGraphCell> connectedCells) {
        Integer hash = getHashCode(noteCell);
        if (connectedCells != null) {
            for (DefaultGraphCell cell: connectedCells) {
                hash += OWLGraphUtils.getHashCode(cell); 
            }
        }
        return hash;

    }
    
    public static String getId(OWLClass owlClass) {
        return (OWLClassCell.SIGNATURE + owlClass.toString());
    }
    
    public static String getId(OWLObjectProperty objectProperty) {
        return (OWLObjectPropertyCell.SIGNATURE + objectProperty.toString());
    }
    
    public static String getId(OWLVertexCell vertex) {
        if (vertex instanceof OWLObjectPropertyCell) {
            return getId((OWLObjectPropertyCell) vertex);
        } else if (vertex instanceof OWLClassCell) {
            return getId((OWLClassCell) vertex);
        } else if (vertex instanceof OWLNoteCell) {
            return getId((OWLNoteCell) vertex);
        }
        return "";
    }
    
    protected static String getId(OWLClassCell owlClass) {
        return getId(OWLClassCell.SIGNATURE, owlClass);
    }
    
    protected static String getId(OWLObjectPropertyCell objectProperty) {
        return getId(OWLObjectPropertyCell.SIGNATURE, objectProperty);
    }
    
    protected static String getId(OWLNoteCell note) {
        return getId(OWLNoteCell.SIGNATURE, note);
    }
    
    protected static String getId(String header, OWLVertexCell vertex) {
        return (header + vertex.getUserObject().toString());
    }
    
}